/*
 *  IpHome
 *
 *  Collin R. Mulliner <collin@betaversion.net>
 *  http://www.mulliner.org/nokia770/
 *
 *  This is GPL software
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <string.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
//#include <hildon-home-plugin/hildon-home-plugin-interface.h>
//#include <hildon-widgets/gtk-infoprint.h>
#include <libosso.h>

#include "net.h"

#define PACKAGE_VERSION "1.0"

//#define IFACE "eth0"
#define IFACE "wlan0"

typedef struct
{
	osso_context_t *ctxt;

	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *hbox2;
	GtkWidget *vbox;
	GtkWidget *iplabel;
	GtkWidget *txlabel;
	GtkWidget *rxlabel;
	GtkWidget *txelabel;
	GtkWidget *rxelabel;

	guint timer;
	
} IpHpHomeCtx;

void* hildon_home_applet_lib_initialize(void *state_data, int *state_size, GtkWidget **applet_return)
{
	IpHpHomeCtx *ctx = g_new0(IpHpHomeCtx, 1);
	
	ctx->frame = gtk_frame_new(NULL);
	gtk_widget_show(ctx->frame);
	*applet_return = ctx->frame;
	gtk_frame_set_shadow_type(GTK_FRAME(ctx->frame), GTK_SHADOW_ETCHED_IN);

	ctx->vbox = gtk_vbox_new(FALSE, 1);
	gtk_widget_show(ctx->vbox);
	gtk_container_add(GTK_CONTAINER(ctx->frame), ctx->vbox);

	ctx->iplabel = gtk_label_new("XXX");
	gtk_widget_show(ctx->iplabel);
	gtk_label_set_justify(GTK_LABEL(ctx->iplabel), GTK_JUSTIFY_FILL);
	gtk_container_add(GTK_CONTAINER(ctx->vbox), ctx->iplabel);

	ctx->hbox = gtk_hbox_new(FALSE, 1);
	gtk_widget_show(ctx->hbox);
	gtk_container_add(GTK_CONTAINER(ctx->vbox), ctx->hbox);
	ctx->txlabel = gtk_label_new("XXX");
	gtk_widget_show(ctx->txlabel);
	gtk_label_set_justify(GTK_LABEL(ctx->txlabel), GTK_JUSTIFY_FILL);
	gtk_container_add(GTK_CONTAINER(ctx->hbox), ctx->txlabel);
	ctx->rxlabel = gtk_label_new("XXX");
	gtk_widget_show(ctx->rxlabel);
	gtk_label_set_justify(GTK_LABEL(ctx->rxlabel), GTK_JUSTIFY_FILL);
	gtk_container_add(GTK_CONTAINER(ctx->hbox), ctx->rxlabel);

	ctx->hbox2 = gtk_hbox_new(FALSE, 1);
	gtk_widget_show(ctx->hbox2);
	gtk_container_add(GTK_CONTAINER(ctx->vbox), ctx->hbox2);
	ctx->txelabel = gtk_label_new("XXX");
	gtk_widget_show(ctx->txelabel);
	gtk_label_set_justify(GTK_LABEL(ctx->txelabel), GTK_JUSTIFY_FILL);
	gtk_container_add(GTK_CONTAINER(ctx->hbox2), ctx->txelabel);
	ctx->rxelabel = gtk_label_new("XXX");
	gtk_widget_show(ctx->rxelabel);
	gtk_label_set_justify(GTK_LABEL(ctx->rxelabel), GTK_JUSTIFY_FILL);
	gtk_container_add(GTK_CONTAINER(ctx->hbox2), ctx->rxelabel);

	ctx->ctxt = osso_initialize ("iphome_home", PACKAGE_VERSION, FALSE, NULL);

	ctx->timer = 0;
	
	//fprintf(stderr, "initialize\n");
	return(ctx);
}

int hildon_home_applet_lib_get_requested_width(IpHpHomeCtx *ctx)
{
	//fprintf (stderr, "get width\n");

	GtkRequisition req;
	gtk_widget_size_request(ctx->frame, &req);

	return(req.width);
}

int hildon_home_applet_lib_save_state(IpHpHomeCtx *ctx, void **state_data, int *state_size)
{
	//fprintf (stderr, "save_state\n");
	return(0);
}

void hildon_home_applet_lib_background(IpHpHomeCtx *ctx)
{
	//fprintf (stderr, "background\n");
	if (ctx->timer) {
		g_source_remove(ctx->timer);
		ctx->timer = 0;
	}
}

gboolean update(IpHpHomeCtx *ctx);

void hildon_home_applet_lib_foreground(IpHpHomeCtx *ctx)
{
	//fprintf (stderr, "foreground\n");
	if (!ctx->timer) {
		update(ctx);
		ctx->timer = g_timeout_add(1000, (GSourceFunc)update, (gpointer)ctx);
	}
}

gboolean update(IpHpHomeCtx *ctx)
{
	gchar *text = NULL;

	//fprintf(stderr, "updated called\n");
	text = g_strdup_printf("%s:\t%s", IFACE, iface_get_addr_string(IFACE));
	gtk_label_set_text(GTK_LABEL(ctx->iplabel), text);
	g_free(text);
	text = g_strdup_printf("RX: %s", iface_get_rxbytes(IFACE));
	gtk_label_set_text(GTK_LABEL(ctx->rxlabel), text);
	g_free(text);
	text = g_strdup_printf("TX: %s", iface_get_txbytes(IFACE));
	gtk_label_set_text(GTK_LABEL(ctx->txlabel), text);
	g_free(text);
	text = g_strdup_printf("Err TX: %s", iface_get_txerr(IFACE));
	gtk_label_set_text(GTK_LABEL(ctx->txelabel), text);
	g_free(text);
	text = g_strdup_printf("RX: %s", iface_get_rxerr(IFACE));
	gtk_label_set_text(GTK_LABEL(ctx->rxelabel), text);
	g_free(text);

	return(TRUE);
}

GtkWidget* hildon_home_applet_lib_properties(IpHpHomeCtx *ctx, GtkWindow *parent)
{
	//fprintf (stderr, "properties\n");
	return(NULL);
}

void hildon_home_applet_lib_deinitialize(IpHpHomeCtx *ctx)
{
	hildon_home_applet_lib_background(ctx);
	osso_deinitialize(ctx->ctxt);
	g_free (ctx);

	//fprintf (stderr, "deinitialize\n");
}
